%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Identification with    %
%   Least Squares method   %  
%                          %
%   date: 24/5/2019        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all, clear all, clc;

disp('+++++++++++++++++++++++++++++++++++++');
disp('|      Method of Least Squares      |');
disp('+++++++++++++++++++++++++++++++++++++');   

% DATA PREPARATION
tSample = 0.001;
tRange = 80;   
t = [0:tSample:tRange]; 
u = sin(3*pi+0.1*t) + sin(2*pi+0.3*t) + sin(3*pi+0.01*t);  

% DEFINITION OF SYSTEM
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nSystem for identification\n');
K = 3;
s = tf('s');
sys = K/((3*s+1)*(2.5*s+1)*(2.5*s+1)) 

% SIMULATING OF APPLICATION OF INPUT SIGNAL INTO A SYSTEM
y = lsim(sys, u, t);

% PREPARING PHI VALUES
Y = y(3:end);
PHI = [-y(2:end-1) -y(1:end-2) u(2:end-1)' u(1:end-2)']; 

% COMPUTING PARAMETERS
THETA = (PHI'*PHI)^-1 * PHI'*Y;

% IDENTIFICATION MODEL
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nDiscrete model\n');
Gz = tf([THETA(3) THETA(4)], [1 THETA(1) THETA(2)], tSample)   
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nContinuous model\n');
Gs = d2c(Gz)   % Discrete model to Continuous

% SIMULATION OF INPUT ON COMPUTED MODEL
yModel = lsim(Gs, u, t);

% PLOTTING
plot(t, u, 'g-.', 'LineWidth', 2);    
hold on;
plot(t, y, 'r-.', 'LineWidth', 2);
plot(t, yModel, 'b', 'LineWidth', 1);
hold off; grid on;
xlim([-0.5 tRange+0.5]); 
legend('input', 'system output', 'LS method model output');      
title('\bf \fontsize{12} Least Squares method identification'); 
xlabel('\bf t [s]'); ylabel('\bf u(t), y(t)');

fprintf('\n');
disp('+++++++++++++++++++++++++++++++++++++');
disp('|                END                |');
disp('+++++++++++++++++++++++++++++++++++++');
