%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Identification         %
%   with Recursive Least   %
%   Squares method         %  
%                          %
%   date: 24/5/2019        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all, clear all, clc;

disp('+++++++++++++++++++++++++++++++++++++');
disp('| Recursive Method of Least Squares |');
disp('+++++++++++++++++++++++++++++++++++++');   

% DATA PREPARATION
tSample = 0.01;
tRange = 80 ;   
t = [0:tSample:tRange]; 
u = sin(3*pi+0.1*t) + sin(2*pi+0.3*t) + sin(3*pi+0.01*t);    

% DEFINITION OF SYSTEM
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nSystem for identification\n');
K = 3; s = tf('s');
sys = K/((3*s+1)*(2.5*s+1)*(2.5*s+1))     

% SIMULATING OF APPLICATION OF INPUT SIGNAL INTO A SYSTEM
y = lsim(sys, u, t);

% PREPARING THETA, COVARIANT MATRIX, LAMBDA, I/O
PCovKoef = 1e6;  % 1e4 - 1e6
PCovariant = PCovKoef*eye(6);
THETA = [1; 0; 0; 0; 0; 0];
THETA_BUFFER = [];
lambda = 0.98;
U = [0 0 0 0];
Y = [0 0 0 0]; 

% COMPUTING PARAMETERS
for i = 1:length(u)
  U=[U u(i)];
  Y=[Y y(i)];
    
  [THETA PCovariant] = rls(U, Y, PCovariant, THETA, lambda);
  THETA_BUFFER = [THETA_BUFFER THETA]; 

end

% IDENTIFICATION MODEL
warning('off','all');
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nDiscrete model 1\n');
Gz = tf([THETA(4) THETA(5) THETA(6)], ...
        [1 THETA(1) THETA(2) THETA(3)], tSample)     
disp('+++++++++++++++++++++++++++++++++++++');
fprintf('\nContinuous model 1\n');
Gs = d2c(Gz)    % Discrete model to Continuous

% SIMULATION OF INPUT ON COMPUTED MODEL
tn = [0:(tSample/100):tRange];          % to avoid undersampling
un = sin(3*pi+0.1*tn) + sin(2*pi+0.3*tn) + sin(3*pi+0.01*tn);
yModel = lsim(Gs, un, tn);


% PLOTTING System and model output
figure(1);
plot(tn, un, 'g-.', 'LineWidth', 2);    
hold on;
plot(t, y, 'r-.', 'LineWidth', 2);
plot(tn, yModel, 'b', 'LineWidth', 1);
hold off; grid on;
xlim([-0.5 tRange+0.5]); 
legend('input', 'system output', 'model ouput');      
title('\bf \fontsize{12} Recursive Least Squares method identification'); 
xlabel('\bf t [s]'); ylabel('\bf u(t), y(t)');

% PLOTTING time changes in parameters
figure(2); hold on;
plot(t, THETA_BUFFER(1, :), 'k');
plot(t, THETA_BUFFER(2, :), 'b');
plot(t, THETA_BUFFER(3, :), 'c');
plot(t, THETA_BUFFER(4, :), 'r');
plot(t, THETA_BUFFER(5, :), 'm');
plot(t, THETA_BUFFER(6, :), 'g');
hold off; grid on;
xlim([-0.5 tRange+0.5]);
legend('a1', 'a2', 'a3', 'b1', 'b2', 'b3');      
title('\bf \fontsize{12} Recursive Least Squares parameters from model.'); 
xlabel('\bf t [s]');

fprintf('\n');
disp('+++++++++++++++++++++++++++++++++++++');
disp('|                END                |');
disp('+++++++++++++++++++++++++++++++++++++');
