%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Identification           %
%   with Spectral Analysis   %  
%                            %
%   date: 24/5/2019          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; clc;

disp('++++++++++++++++++++++++++++++++');
disp('|       Spectral Anaysis       |');
disp('++++++++++++++++++++++++++++++++');   

% DATA PREPARATION
tSample = 0.01;
tRange = 2000;
t = [0:tSample:tRange];

%Generating of PRBS signal, expected value (mean) = 0
try
    warning('off','all');
    u = idinput(length(t), 'PRBS', [0 1], [-3 3]);
    warning('on','all');
catch
    PRBS = @(N) randi([-1 1], 1, N);
    u = PRBS(length(t));
end

% DEFINITION OF THE SYSTEM
disp('++++++++++++++++++++++++++++++++');
fprintf('\nSystem for identification\n');
s = tf('s');
K = 100; T = 5;
sys = K/(T*s+1)

% SIMULATING OF APPLICATION OF INPUT SIGNAL INTO A SYSTEM
y = lsim(sys, u, t); 

% FOURIER TRANSFORM (cross power spectral density)
% FROM AUTOCORRELATION AND CORRELATION
[FIuu, u_freq] = cpsd(u, u, [], [], [], 1/tSample);  
[FIuy, ~] = cpsd(u, y, [], [], [], 1/tSample);

% IDENTIFICATION MODELS - frequency response characteristic
% (complex number)
G = FIuy./FIuu;

% PLOTTING
figure(1);   % plot input and output signals
subplot(2, 1, 1); grid on;
plot(t, u, 'b');  
xlim([0 tRange*0.01 + 0.5]);
title('\bf \fontsize{12} Input signal - PRBS');
xlabel('\bf t [s]'); ylabel('\bf u(t)');

subplot(2, 1, 2); grid on;
plot(t, y, 'r');  
xlim([0 tRange*0.01 + 0.5]);
title('\bf \fontsize{12} Output signal - PRBS response');
xlabel('\bf t [s]'); ylabel('\bf y(t)');

figure(2); % plot Amplitude freq. response characteristic

% multiply freq. by 2pi to get angular frequency:
semilogx(u_freq*2*pi, 20*log10(abs(G)), 'r', 'linewidth', 3);    
hold on; grid on;
title(['\bf \fontsize{12} Amplitude freq. response' ...
       ' characteristic from model']);
xlabel('\omega [1/s]'); ylabel('|G(j\omega)| [dB]');

fprintf('\n');
disp('++++++++++++++++++++++++++++++++');
disp('|             END              |');
disp('++++++++++++++++++++++++++++++++');
